// Buhai, Portela, Teulings, and van Vuuren: Returns to Seniority or Tenure?
#include <oxstd.h>
#import <maximize>
#include <packages/oxmpi/oxmpi.h>

class Data_hulp {
	public : decl X0, D, X, NSize, Y, Y_Firm;
	
	Data_hulp(const x0, const x, const nsize, const y, const y_firm, const d);
	Get_X0();
}

Data_hulp::Data_hulp(const x0, const x, const nsize, const y, const y_firm, const d)
{
		X0  = x0;
		X = x;
		NSize = nsize;
		Y = y;
		Y_Firm = y_firm;
		D = d;
}

Data_hulp::Get_X0()
{
		return X0;
}

class ModelBase {
	// per spell data
	
		decl 	X,	// rank per spell
				Y, // other regressors per spell
				Firm, // Firm identifier per spell
				NSize,	// d_firmsize per spell
				Y_Firm,		// regressors for the correlated random effects of firms
				Chi,  // firm random effect per spell
				Amt, // Location per spell
				Age, // age at entry per spell
				Educ, // education level per spell
				Chi1,	   // firm random effects 
				D, // Dummy for right censored per spell
				Sec, // Sector per spell
				T;// duration per spell

	// overall data			
		decl 	X1, // rank for all spells
				T1,	   // durations over all spells
				D1, // dummy for right censored over all spells
				Educ_Firm, // average education level in the firm
				Age_Firm, // average age in the firm
				NSize1, // d_firmsize for all spells
				Women_Firm;  // t_start for every spell
			
			// Parameters to be estimated			
			
		decl 	Gamma, // Coefficient of Y			
				K_firm, // number of regressors for correlated random effects of firms
				Beta,// coefficient for rank
				Beta1, // coefficient for d_firmsize			
				Sigma_chi, // standard deviation of firm random effects
				Psi, // set of variables for the piecewise constant hazard
				V0,
				V1,
				V2,
				V3,
				P0,
				P1,
				P2,
				P3,
				P,
				Hulp_0,
				X_hulp_0,
				Xi; 
			
			
// additional parameters/variables			
		decl	Ran_hulp,  
				Max_Value,
				Amt_Array,	// Array for the location numbers (baseline not included)
				Sec_Array, // Array for the sector numbers (baseline not included)
				Tot_files,	// total number of files			
				n_firms, // highest firm identifier 									
				K, // number of other regressors	
				N,	  // number of observations per file
				K2,	 // Number of spells used
				P_random, // number of draws from normal distribution to obtain the integral for the random effects of firms
				K1; // size of Psi


	public:  decl myid;
	decl	numprocs,
				procname;

	static decl started = 0;
	static decl finished = 0;

	ModelBase();
	~ModelBase();
	virtual FLoglik(const vp, const adfunc, const avscore, const mhess);
	virtual FLoglik10(const vP, const adFunc, const avScore, const mHess);
	virtual FLoglik1a(const vP, const adFunc, const avScore, const mHess);
	virtual FLoglik2a(const vP, const adFunc, const avScore, const mHess);
	virtual FLoglik10b(const vP);
	protected : HandleData(const i, const hulp2);
	protected : HandleData1(const i, const hulp2);
	protected : Read_And_Handle_Files(const i);
	protected : Handle_FLoglik(const vP, const adFunc, const avScore, const adfunc0, const avscore0);
	protected : Handle_FLoglik1(const vP, const adFunc, const adfunc0);
	protected : Handle_FLoglik10b(const adfunc0, const avscore0);
	virtual protected	: BerekenData1a(const vP);
	virtual protected: BerekenData(const vP);
	BerekenStandardErrors(const vP);
}


ModelBase::ModelBase()
{
	if (started == 0)
	{
		OxMPI_Init();
		started = 1;
	}
   	numprocs = OxMPI_Comm_size();
	myid = OxMPI_Comm_rank();
   	procname = OxMPI_Get_processor_name();	
	
	P_random = 100;
	Amt_Array = range(1,11,1)'; // Array for the dummies to be created for region
	Sec_Array = range(1,11,1)'; // Array for the dummies to be created for sector
	Tot_files = 88;	// NUMBER OF FILES;
	K_firm = 3;
	n_firms = 1999;
	K = 5+11+11; 
	Max_Value = -1E+30;
	K1 = 22;
	K2=10;

	if (Tot_files == numprocs)
	{
		Read_And_Handle_Files(myid+1);
	}
}


ModelBase::~ModelBase()
{
	if (finished == 0)
	{
		OxMPI_Finalize();
		finished = 1;
	}
}

ModelBase::BerekenStandardErrors(const vP)
{
	decl avscore = FLoglik10b(vP);

	decl k_firm = K_firm;
	decl k = K;
	decl k1 = K1;

	if (myid .== 0)
	{
		savemat("avscore1_2m_baseline_new.fmt", avscore);
		decl avscore1 = avscore'*avscore;
		savemat("avscore1_2m_baseline_new.xls", avscore1);
		decl cov = invert(avscore1);

		return sqrt(diagonal(cov)');
	}
}

 
ModelBase::FLoglik1a(const vP, const adFunc, const avScore, const mHess)
{
	decl v = vP[K+K_firm+K1+1];
	
	decl n1 = rows(X);
	decl t1 = sumr(!(X .== -1));
	
	decl exp_hul0 = X_hulp_0.Get_X0() .* exp(v+Chi);
	decl theta0 = exp_hul0 ./ (1 + exp_hul0);
	theta0 = theta0 .* !(X.==-1);
	decl theta1 = 1 - theta0;

	decl hulp6 = prodc((theta1)')';
	decl hulp2 = hulp6 .* diagonal(exp_hul0[][t1-1])';
	

	decl floglik4 = hulp2 .* (1-D) + D .* hulp6;

	adFunc[0] = floglik4;
	
	if (avScore)
	{
		decl dtheta0_1 = theta1 .* theta0;
		decl dtheta1_1 = theta1;		
		decl dhulp1 = -sumc(theta0')';
		decl dhulp2 = dhulp1 + 1;

		decl dx1b = X .* (!(X .== -1));
		decl dx1c = NSize .* (!(NSize .== -1));

		decl dhulp3 = theta0 .* dx1b;		
		decl dhulp3_a = theta0 .* dx1c;
		decl dhulp4 = -sumc(dhulp3')';		
		decl dhulp4a = -sumc(dhulp3_a')';
	
		decl dhulp5 = dhulp4 + diagonal(dx1b[][t1-1])';
		decl dhulp5a =dhulp4a +  diagonal(dx1c[][t1-1])';

		decl avscore4 = (dhulp2 .* (1-D) + D .* dhulp1);
		decl avscore1 = (dhulp5 .* (1-D) + dhulp4 .* D);
		decl avscore1a = (dhulp5a .* (1-D) + dhulp4a .* D);
		decl avscore2 = avscore4  .* Y;
		decl avscore2a = avscore4  .* Y_Firm;
		decl dhulp15 = -theta0 .* (t1 .>= range(1,K1,1) .* ones(n1,K1));
		decl dhulp14 = dhulp15 + (t1 .== range(1,K1,1));
		decl avscore3 = (dhulp14 .* (1-D) + D .* dhulp15);		
		

		decl avScore_0 =  (avscore1~avscore1a~avscore2~avscore2a~avscore3[][1:K1-1]~avscore4~avscore4);
		
		avScore[0] = avScore_0;
	}

	return 1;
}


ModelBase::BerekenData1a(const vP)
{
	Psi = 0|vP[K+K_firm+2:K+K_firm+K1];
	decl v = vP[K+K_firm+K1+1];
	
	decl i, hulp10 = <>, hulp12 = <>, hulp13 = <>, hulp100 = <>;
	decl x1 = X~-ones(rows(X),1);
	decl nsize = NSize~-ones(rows(NSize),1);
	decl psi10 = Psi|ones(K1-rows(Psi),1) .* Psi[rows(Psi)-1];
	decl n1 = rows(T);
	decl x1a = Y * Gamma;
	decl x1a_b = Y_Firm * Xi;
	decl x1b = X * Beta .* (!(X .== -1));
	decl x1c = NSize * Beta1 .* (!(NSize .== -1));
	decl t1 = sumr(!(X .== -1));

	decl x0 = new Data_hulp(exp(x1a + x1a_b + x1b + x1c + psi10'), X, NSize, Y, Y_Firm, D);
	return x0;
}


ModelBase::HandleData(const i, const hulp2)
{
	X_hulp_0 = Hulp_0[i];
	X = Hulp_0[i].X;
	NSize = Hulp_0[i].NSize;
	Y = Hulp_0[i].Y;
	Y_Firm = Hulp_0[i].Y_Firm;
	D = Hulp_0[i].D;

	decl firm = selectifr(Firm[][i], T1[][i] .>= 0);
			
	Chi = Chi1[(firm -1) /*.* (firm .> 0)*/];
	decl dchi = Ran_hulp[(firm-1) /*.* (firm .> 0)*/];
 
	return dchi;
}


ModelBase::HandleData1(const i, const hulp2)
{
	X = selectifr(X1[][i .*K1:K1 .* (i+1) -1], T1[][i] .>= 0);	
	NSize = selectifr(NSize1[][i .*K1:K1 .* (i+1) -1], T1[][i] .>= 0);
	Y = selectifr(Educ[][i], T1[][i] .>= 0);
	decl exp = Age[][i]-Educ[][0]-6;
	decl exp2 = exp .* exp / 100;
	decl exp3 = exp .* exp2 / 100;
	decl exp4 = exp .* exp3 / 100;
	Y =Y~ selectifr(exp, T1[][i] .>= 0);
	Y = Y~ selectifr(exp2, T1[][i] .>= 0);
	Y = Y~ selectifr(exp3, T1[][i] .>= 0);
	Y = Y~ selectifr(exp4, T1[][i] .>= 0);

	decl index;
	for(index=0;index<rows(Amt_Array);index++)
	{
		Y = Y~selectifr(Amt[][i] .== Amt_Array[index], T1[][i] .>= 0);
	}

	for(index=0;index<rows(Sec_Array);index++)
	{
		Y = Y~selectifr(Sec[][i] .== Sec_Array[index], T1[][i] .>= 0);
	}
		
		
	Y_Firm = selectifr(Women_Firm[][i], T1[][i] .>= 0);
	Y_Firm = Y_Firm~selectifr(Educ_Firm[][i], T1[][i] .>= 0);
	Y_Firm = Y_Firm~selectifr(Age_Firm[][i]-Educ_Firm[][0]-6, T1[][i] .>= 0);

	D = selectifr(D1[][i], T1[][i] .>= 0);
	T = selectifr(T1[][i], T1[][i] .>= 0);

	   
	return 1;
}

/* Computes likelihood for all spells per individual, given the unobserved heterogeneity */

ModelBase::BerekenData(const vP){
	
	decl i;
	Hulp_0 = {};
	
	for(i=0;i<K2;i++)
	{
		decl hulp2;
		HandleData1(i, &hulp2);

		decl hulp, dhulp;

		Hulp_0 = Hulp_0|BerekenData1a(vP);
	}

	return 1;
}

ModelBase::FLoglik2a(const vP, const adFunc, const avScore, const mHess)
{
	decl floglik = ones(N,1), dfloglik = zeros(N,rows(vP)), i;

			
	for(i=0;i<K2;i++)
	{
		decl hulp2;
		decl dchi = HandleData(i, &hulp2);

		decl hulp, dhulp;

		if (rows(X) .> 0)
			FLoglik1a(vP, &hulp, 0, 0);
		else
			hulp = <>;
			
		if (avScore)
		{
			if (rows(X) .> 0)
			{
				FLoglik1a(vP, &hulp, &dhulp, 0);
			}
			else
			{
				hulp = <>;
				dhulp = <>;
			}
		}

		decl avscore, avscore1;
					  
		hulp = hulp|ones(N-rows(hulp),1);
	
		if (avScore)
		{				   				
			dchi = dchi|zeros(N-rows(vec(dchi)),1);
			dhulp = dhulp|zeros(N-rows(dhulp),rows(vP));
		}
		
		floglik = floglik .* hulp;

		if (avScore)
		{		
			dhulp[][columns(dhulp)-1] = dhulp[][columns(dhulp)-1] .* dchi;
			dfloglik = dfloglik + dhulp;
		}
	}

	adFunc[0] = floglik;

	if (avScore)
	{
		avScore[0] = dfloglik;
	}

	return 1;
}


ModelBase::FLoglik10b(const vP)
{
	decl i;

   decl avscore0 = <>;

	decl n1 = 0;
	decl adfunc0 = <>;

	for(i=myid+1;i<=Tot_files;i+=numprocs)
	{
		Read_And_Handle_Files(i);
		decl adfunc, avscore;
		decl avscore1;
		FLoglik(vP, &adfunc, &avscore, 0);
		adfunc0 = adfunc0|adfunc;
		avscore0 = avscore0|avscore;


		n1 +=  rows(D1);
	}

	return Handle_FLoglik10b(adfunc0, avscore0);
}

ModelBase::Handle_FLoglik10b(const adfunc0, const avscore0)
{
	decl adfunc1 = <>, avscore1;
	decl i;
	avscore1 = <>;
	if (myid == 0)
	{
		adfunc1 = adfunc0;
			avscore1 = avscore0;
		for(i=1;i<numprocs;i++)
		{
			adfunc1 = adfunc1|OxMPI_Recv(i,0);
				avscore1  = avscore1|OxMPI_Recv(i, 1);
		}
	}
	else
	{
		OxMPI_Send(adfunc0, 0, 0);		
		OxMPI_Send(avscore0, 0, 1);
	}

	if (myid == 0)
  	{			
			return avscore1;
 	}	 
}

ModelBase::Read_And_Handle_Files(const i)
{
	decl i1 = i - 1;
		decl filestr = sprint("x1_intermed_", i, ".fmt");
		decl filestr1 = sprint("d1_intermed_", i, ".fmt");
		decl filestr2 = sprint("t2_intermed_", i, ".fmt");
		decl filestr4 = sprint("educ_intermed_", i, ".fmt");
		decl filestr5 = sprint("amt_intermed_", i, ".fmt");
		decl filestr6 = sprint("sec_intermed_", i, ".fmt");
		decl filestr8=sprint("age_intermed_", i, ".fmt");
		decl filestr10 =sprint("dsize_intermed_", i,".fmt");
		decl filestr11 =sprint("women_firm_intermed_", i,".fmt");
		decl filestr12 =sprint("educ_firm_intermed_", i,".fmt");
		decl filestr13 =sprint("age_firm_intermed_", i,".fmt");
		decl filestr14 =sprint("firm_intermed_", i,".fmt");
	
		X1 = log(loadmat(filestr));
		D1 = loadmat(filestr1);
		T1 = loadmat(filestr2);
		Age = loadmat(filestr8);
		Educ = loadmat(filestr4);
		Amt = loadmat(filestr5);
		Sec = loadmat(filestr6);
		NSize1 = loadmat(filestr10);
		Women_Firm = loadmat(filestr11);
		Educ_Firm = loadmat(filestr12);
		Age_Firm = loadmat(filestr13);
		Firm = loadmat(filestr14);

		

		//***************************
		  
		decl hulp = T1 .>= 0;
		decl hulp1 = sumr(hulp);
						 
		X1 = sortbyc(hulp1~X1,0);
		D1 = sortbyc(hulp1~D1,0);
		T1 = sortbyc(hulp1~T1,0);
		Educ = sortbyc(hulp1~Educ,0);
		Amt = sortbyc(hulp1~Amt,0);
		Sec = sortbyc(hulp1~Sec,0);
		NSize1 = sortbyc(hulp1~NSize1,0);
		Women_Firm = sortbyc(hulp1~Women_Firm,0);
		Educ_Firm = sortbyc(hulp1~Educ_Firm,0);
		Age_Firm = sortbyc(hulp1~Age_Firm,0);
		Firm = sortbyc(hulp1~Firm,0);
		Age=sortbyc(hulp1~Age,0);
		
		X1 = X1[][1:];
		D1 = D1[][1:];
		T1 = T1[][1:];
		Educ = Educ[][1:];
		Amt = Amt[][1:];
		Sec = Sec[][1:];
		Age=Age[][1:];
		NSize1=NSize1[][1:];
		Women_Firm=Women_Firm[][1:];
		Educ_Firm=Educ_Firm[][1:];
		Age_Firm=Age_Firm[][1:];
		Firm=Firm[][1:];
		
		X1 = reversec(X1);
		D1 = reversec(D1);
		T1 = reversec(T1);
		Educ = reversec(Educ);
		Amt = reversec(Amt);
		Sec = reversec(Sec);
		Age=reversec(Age);
		NSize1=reversec(NSize1);
		Women_Firm=reversec(Women_Firm);
		Educ_Firm=reversec(Educ_Firm);
		Age_Firm=reversec(Age_Firm);
		Firm=reversec(Firm);
						   
		N = rows(X1);

		return hulp1;
}


/* FLoglik10 reads in the files and starts the likelihood procedure per file. In the end the Floglik per file is collected and returned. The
sorting is to speed up things. It sorts based on the number of spells. */
ModelBase::FLoglik10(const vP, const adFunc, const avScore, const mHess)
{
	decl i;

   decl avscore0;
	if (avScore)
		avscore0 = <>;

	decl n1 = 0;
	decl adfunc0 = <>;

	for(i=myid+1;i<=Tot_files;i+=numprocs)
	{
		if (!(numprocs == Tot_files))
		{
			Read_And_Handle_Files(i);
			//println("reached line 500");
		 }
		decl adfunc, avscore;
		if (avScore)
		{
			decl avscore1;
			FLoglik(vP, &adfunc, &avscore, 0);
			adfunc0 = adfunc0|adfunc;
			avscore0 = avscore0|avscore;
		}
		else
		{
			FLoglik(vP, &adfunc, 0, 0);
			adfunc0 = adfunc0|adfunc;
		}

		n1 +=  rows(D1);
	}

	if (avScore)
		Handle_FLoglik(vP, adFunc, avScore, adfunc0, avscore0);
	else
		Handle_FLoglik1(vP, adFunc, adfunc0);

	return 1;
}

ModelBase::Handle_FLoglik(const vP, const adFunc, const avScore, const adfunc0, const avscore0)
{
	decl adfunc1 = <>, avscore1;
	decl i;
	if (avScore)
		avscore1 = <>;
	if (myid == 0)
	{
		adfunc1 = adfunc0;
		if (avScore)
		{
			avscore1 = avscore0;
		}
		for(i=1;i<numprocs;i++)
		{
			adfunc1 = adfunc1|OxMPI_Recv(i,0);
			if (avScore)
			{
				avscore1  = avscore1|OxMPI_Recv(i, 1);
			}			
		}
	}
	else
	{
		OxMPI_Send(adfunc0, 0, 0);
		if (avScore)
		{
			OxMPI_Send(avscore0, 0, 1);
		}
	}

	if (myid == 0)
  	{ 
		adFunc[0] = meanc(adfunc1);

		if (avScore)
		{
			if (adFunc[0] .> Max_Value) {
				savemat("param_intermed2_2m_new.xls", vP);
				Max_Value = adFunc[0];				
			}
			
			avScore[0] = meanc(avscore1)';
		}
		
		for(i=0;i<numprocs;i++)
		{
			OxMPI_Send(adFunc[0], i, 2);
			if (avScore)
			{
				OxMPI_Send(avScore[0], i, 3);			
			}
		}
		  
		if (adFunc[0] .> Max_Value) {
			savemat("param_intermed_2m_baseline_new.xls", vP);
		}
   	}
	else
	{
		decl adfunc2 = OxMPI_Recv(0, 2);
		adFunc[0] = adfunc2;
		if (avScore)
		{
			decl avscore2 = OxMPI_Recv(0,3);
			avScore[0] = avscore2;
		}
	}

	if (myid .== 0)
	{
		println(adFunc[0]);
	}	  
}

ModelBase::Handle_FLoglik1(const vP, const adFunc, const adfunc0)
{
	decl adfunc1 = <>;
	decl i;
	if (myid == 0)
	{
		adfunc1 = adfunc0;
		for(i=1;i<numprocs;i++)
		{
			adfunc1 = adfunc1|OxMPI_Recv(i,0);
		}
	}
	else
	{
		OxMPI_Send(adfunc0, 0, 0);
	}

	if (myid == 0)
  	{ 
		adFunc[0] = meanc(adfunc1);

	
		
		for(i=0;i<numprocs;i++)
		{
			OxMPI_Send(adFunc[0], i, 2);
		}
		  
		if (adFunc[0] .> Max_Value) {
			savemat("param_intermed_2m_baseline_new.xls", vP);
		}
    }
	else
	{
		decl adfunc2 = OxMPI_Recv(0, 2);
		adFunc[0] = adfunc2;
	}

	if (myid .== 0)
	{
		println(adFunc[0]);
	}	  
}
   
/* Main likelihood procedure. Calculatates the likelihood and the first order deriative in for all different mass points */ 
ModelBase::FLoglik(const vP, const adFunc, const avScore, const mHess)
{
	println("Please implement this part: FLoglik");	
	
	return 1;
}
 

 