//////////////////////////////////////////////////////////
//	ECMA: MS# 8688-4									//
//	Data, Denmark										//
//	September, 2013										//
//	Buhai, I. S., Portela, M.A, Teulings, C.N., van Vuuren, A.	//
//														//
//	TABLE V												//
//														//
//////////////////////////////////////////////////////////
local st = "$S_TIME"

clear all
	set rmsg on
	set more off
	set matsize 2000

cd "O:\workdata\701835\Sebastian\2ndRevisionseniority\files"
sysdir set PERSONAL "O:\workdata\701835\Sebastian\Useful_stata_ado_files\personal\"

capture log close
	log using table_V.log, text replace
	
	// TOPEL //

	use "O:\workdata\701835\Sebastian\seniorityproject\wageregsdata" if kon2==1, clear 

	tsset id year

// GENERATE FIRST-DIFFERENCED VARIABLES
foreach v of varlist lhwreal tenure tenure2 tenure3 tenure4 pexper2 pexper3 pexper4 logrankns logrank lognsize {
di _new "Variable:	`v'" _new
	gen d`v'=d.`v'	// WITHIN SPELL FIRST-DIFFERENCES
	di _new
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////
// 2nd ORDER POLYNOMIAL & CROSS TERMS //
////////////////////////////////////////

gen lognsize2 = lognsize^2
	gen dlognsize2 = d.lognsize2

	gen dlogrankns2 = d.logrankns2

gen lognsize3 = lognsize^3
	gen dlognsize3 = d.lognsize3
gen logrankns3 = logrankns^3
	gen dlogrankns3 = d.logrankns3

gen cross11 = logrankns*lognsize
	gen dcross11 = d.cross11

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
* # C. TOPEL'S (5) 1ST STEP, INCLUDING LOGRANKNS AND LOGNSIZE

di _new _new _new "2nd ORDER POLYNOMIAL & CROSS TERMS; TOPEL; USUAL PROCEDURE; WITH LOGRANKNS & LOGNSIZE" _new _new

// # 1. USUAL PROCEDURE
tsset id year
reg d.lhwreal d.tenure d.tenure2 d.tenure3 d.tenure4 d.pexper2 d.pexper3 d.pexper4 d.logrankns d.logrankns2 d.lognsize d.lognsize2 d.cross11 y3-y22, nocons

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////
// 2 LAGS OF r AND n //
///////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
* # C. TOPEL'S (5) 1ST STEP, INCLUDING LOGRANKNS AND LOGNSIZE

di _new _new _new "2 LAGS OF r AND n; TOPEL; USUAL PROCEDURE" _new _new

// # 1. USUAL PROCEDURE
tsset id year
reg d.lhwreal d.tenure d.tenure2 d.tenure3 d.tenure4 d.pexper2 d.pexper3 d.pexper4 d.logrankns l.d.logrankns l2.d.logrankns d.lognsize l.d.lognsize l2.d.lognsize y5-y22, nocons

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////
// EXCLUDE HIGHEST 25% OF CHANGES IN q //
/////////////////////////////////////////

	replace dlogrank = -dlogrank
	sum dlogrank, detail
	
	drop if dlogrank > r(p75) & dlogrank ~= .

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// TOPEL

		// TOPEL 2.

		tsset id year
		reg d.lhwreal d.tenure d.tenure2 d.tenure3 d.tenure4 d.pexper2 d.pexper3 d.pexper4 d.logrankns d.lognsize y3-y22, nocons
			global b1b2 = _b[d.tenure]
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Altonji & Shakotko //

	use "O:\workdata\701835\Sebastian\seniorityproject\wageregsdata" if kon2==1, clear 

	tsset id year

// GENERATE FIRST-DIFFERENCED VARIABLES
foreach v of varlist lhwreal tenure tenure2 tenure3 tenure4 pexper2 pexper3 pexper4 logrankns logrank lognsize {
di _new "Variable:	`v'" _new
	gen d`v'=d.`v'	// WITHIN SPELL FIRST-DIFFERENCES
	di _new
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

*****************************
*** Altonji and Shakotko ***
*****************************
gen cross11 = logrankns*lognsize
gen logrankns3 = logrankns^3
gen lognsize2 = lognsize^2
gen lognsize3 = lognsize^3

keep dlogrank cross11 logrank logrankns logrankns2 logrankns3 lognsize lognsize2 lognsize3 idtrab id idemp year lhwreal iniexp tenure tenure2 tenure3 tenure4 pexper pexper2 pexper3 pexper4 educ educ2 educ3 educ4 loc1 loc2 loc4 loc5 loc6 ind2-ind11 y1-y22


	

	tsset
gen laglogrankns = l.logrankns


gen lag2logrankns = l2.logrankns

	
gen laglognsize = l.lognsize


gen lag2lognsize = l2.lognsize


	drop mlag*

// 2nd ORDER POLYNOMIAL & CROSS PRODUCTS
// # AS 4.
xtreg lhwreal tenure tenure2 tenure3 tenure4 logrankns logrankns2 lognsize lognsize2 cross11 pexper pexper2 pexper3 pexper4 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12 y3-y22, fe cluster(idtrab)
// LAG
// # AS 5.
xtreg lhwreal tenure tenure2 tenure3 tenure4 logrankns laglogrankns lag2logrankns lognsize laglognsize lag2lognsize pexper pexper2 pexper3 pexper4 xtreg lhwreal tenure tenure2 tenure3 tenure4 logrankns lognsize pexper pexper2 pexper3 pexper4 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12 y3-y22, fe cluster(idtrab) y5-y22, fe cluster(idtrab)
/////////////////////////////////////////
// EXCLUDE HIGHEST 25% OF CHANGES IN q //
/////////////////////////////////////////
	preserve

	sum dlogrank, detail
		drop if dlogrank > r(p75) & dlogrank ~= .

	xtreg lhwreal tenure tenure2 tenure3 tenure4 logrankns lognsize pexper pexper2 pexper3 pexper4 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12 y3-y22, fe cluster(idtrab)
/////////////////////////////////////////////////////////////////////////////////////////////////

// TOPEL FE //
		
	use "O:\workdata\701835\Sebastian\seniorityproject\wageregsdata" if kon2==1, clear 
	
	tsset id year

// GENERATE FIRST-DIFFERENCED VARIABLES
foreach v of varlist lhwreal tenure tenure2 tenure3 tenure4 pexper2 pexper3 pexper4 logrankns logrank lognsize {
di _new "Variable:	`v'" _new
	gen d`v'=d.`v'	// WITHIN SPELL FIRST-DIFFERENCES
	di _new
}

gen lognsize2 = lognsize^2
	gen dlognsize2 = d.lognsize2

	gen dlogrankns2 = d.logrankns2

gen lognsize3 = lognsize^3
	gen dlognsize3 = d.lognsize3
gen logrankns3 = logrankns^3
	gen dlogrankns3 = d.logrankns3

gen cross11 = logrankns*lognsize
	gen dcross11 = d.cross11



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////
// 2nd ORDER POLYNOMIAL & CROSS TERMS //
////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
di _new _new _new "2nd ORDER POLYNOMIAL & CROSS TERMS; TOPEL; SPELL FE; LOGRANKNS & LOGNSIZE" _new _new

* # 2. 1st stage with SPELL FE

	xtreg dlhwreal dtenure2 dtenure3 dtenure4 dlogrankns dlogrankns2 dlognsize dlognsize2 dcross11 dpexper2 dpexper3 dpexper4 y3-y22, fe cluster(idtrab)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////
// 2 LAGS OF r AND n //
///////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

di _new _new _new "2 LAGS OF r AND n; TOPEL; SPELL FE; WITH LOGRANKNS & LOGNSIZE" _new _new

* # 2. 1st stage with SPELL FE
	
	xtreg dlhwreal dtenure2 dtenure3 dtenure4 dlogrankns ldlogrankns l2dlogrankns dlognsize ldlognsize l2dlognsize dpexper2 dpexper3 dpexper4 y5-y22, fe cluster(idtrab)

//////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////
// EXCLUDE HIGHEST 25% OF CHANGES IN q //
/////////////////////////////////////////

	replace dlogrank = -dlogrank
	sum dlogrank, detail
	
	drop if dlogrank > r(p75) & dlogrank ~= .

		// # TOPELFE 2.

	xtreg dlhwreal dtenure2 dtenure3 dtenure4 dlogrankns dlognsize dpexper2 dpexper3 dpexper4 y3-y22, fe cluster(idtrab)



elapse `st'
log close
