//////////////////////////////////////////////////////////
//	ECMA: MS# 8688-4									//
//	Data, Denmark									//
//	September, 2013										//
//	Buhai, I. S., Portela, M.A, Teulings, C.N, van Vuuren, A.	//
//														//
//	TABLES III, IV & VIII								//
//														//
//////////////////////////////////////////////////////////
local st = "$S_TIME"

clear all
	set rmsg on
	set more off
	set matsize 2000

cd "O:\workdata\701835\Sebastian\2ndRevisionseniority\files"
sysdir set PERSONAL "O:\workdata\701835\Sebastian\Useful_stata_ado_files\personal\"

capture log close
	log using tables_III_IV_VIII.log, text replace

	use "O:\workdata\701835\Sebastian\seniorityproject\wageregsdata" if kon2==1, clear 
	
        tsset id year

	egen t0 = min(year),by(id)
	
	

// GENERATE FIRST-DIFFERENCED VARIABLES
foreach v of varlist lhwreal tenure tenure2 tenure3 tenure4 pexper2 pexper3 pexper4 logrankns logrank lognsize {
di _new "Variable:	`v'" _new
	gen d`v'=d.`v'	// WITHIN SPELL FIRST-DIFFERENCES
	di _new
}

///////////////////////////////////////////////////////////////////////////////////////////////////////

// TABLES III & IV //

///////////////////////////////////////////////////////////////////////////////////////////////////////

// # 1. OLS
di _new _new _new "OLS; WITH just LOGNSIZE" _new _new
	reg lhwreal tenure tenure2 tenure3 tenure4 pexper pexper2 pexper3 pexper4 lognsize y2-y22 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12

// # 1. OLS
di _new _new _new "OLS; WITH LOGRANK & LOGNSIZE" _new _new
	reg lhwreal tenure tenure2 tenure3 tenure4 pexper pexper2 pexper3 pexper4 logrankns lognsize y2-y22 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12

// # 3. //

* # C. TOPEL'S (5) 1ST STEP, INCLUDING JUST LOGNSIZE

di _new _new _new "TOPEL; USUAL PROCEDURE; WITH just LOGNSIZE" _new _new

// # 1. USUAL PROCEDURE
tsset id year
reg d.lhwreal d.tenure d.tenure2 d.tenure3 d.tenure4 d.pexper2 d.pexper3 d.pexper4 d.lognsize y3-y22, nocons
	mat l e(V), format(%16.13g)
	global b1b2 = _b[d.tenure]
	mat omega = e(V)

gen fwage=lhwreal-_b[d.tenure]*tenure-_b[d.tenure2]*tenure2-_b[d.tenure3]*tenure3-_b[d.tenure4]*tenure4 /*
	*/ -_b[d.pexper2]*pexper2 -_b[d.pexper3]*pexper3-_b[d.pexper4]*pexper4-_b[d.lognsize]*lognsize /*
	*/ -_b[y3]*y3-_b[y4]*y4-_b[y5]*y5-_b[y6]*y6-_b[y7]*y7-_b[y8]*y8-_b[y9]*y9-_b[y10]*y10-_b[y11]*y11-_b[y12]*y12-_b[y13]*y13-_b[y14]*y14 /*
	*/ -_b[y15]*y15-_b[y15]*y15-_b[y16]*y16-_b[y17]*y17-_b[y18]*y18-_b[y19]*y19-_b[y20]*y20-_b[y21]*y21-_b[y22]*y22

* SECOND STEP ESTIMATION
reg fwage iniexp t0 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
mat l e(V), format(%16.13g)
	matrix omega1 = e(V)
	global b1 = _b[iniexp]
	global bt0 = _b[t0]
// Standard-error (Murphy & Topel, 1985)	
	quietly reg tenure iniexp t0 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
	mat l e(V), format(%16.13g)
		mat ZT = e(b)'
		mat l ZT, format(%16.13g)
	local vv = "tenure2 tenure3 tenure4 pexper2 pexper3 pexper4 lognsize y3 y4 y5 y6 y7 y8 y9 y10 y11 y12 y13 y14 y15 y16 y17 y18 y19 y20 y21 y22"
		foreach v of local vv {
		di _new "VAR:	`v'"_new
			quietly reg `v' iniexp t0 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
			mat l e(V), format(%16.13g)
				mat ZT = ZT,(e(b)')
				mat l ZT, format(%16.13g)
		}

mat help_matrix = ZT*omega*ZT'
mat l help_matrix, format(%16.13g)
	mat omega_total = omega1 + help_matrix
	mat l omega_total, format(%16.13g)
	matrix omega_total1 = omega_total[1,1]
	mat l omega_total1, format(%16.13g)
		di omega_total1[1,1]^.5
	mat topel_betasi = (($b1)\(omega_total1[1,1]^.5))
	mat l topel_betasi, format(%16.13g)
	matrix help_matrix1 = ZT*omega
	mat l help_matrix1, format(%16.13g)
di "Covariance Omega"				 

	matrix help_matrix2 = help_matrix1[1,1]
	matrix list help_matrix2, format(%16.13g)

	di _new "Return to tenure =    " $b1b2 - $b1 - $bt0 _new
	di _new "Standard-error =	" (omega_total[1,1] + omega[1,1] - 2 * help_matrix2[1,1])^.5

	mat topel_betas = (($b1b2 - $b1)\((omega_total[1,1] + omega[1,1] - 2 * help_matrix2[1,1])^.5))
	mat l topel_betas, format(%16.13g)
drop fwage

// # 4. //

* # C. TOPEL'S (5) 1ST STEP, INCLUDING LOGRANKNS AND LOGNSIZE

di _new _new _new "TOPEL; USUAL PROCEDURE; WITH LOGRANKNS & LOGNSIZE" _new _new

// # 1. USUAL PROCEDURE
tsset id year
reg d.lhwreal d.tenure d.tenure2 d.tenure3 d.tenure4 d.pexper2 d.pexper3 d.pexper4 d.logrankns d.lognsize y3-y22, nocons
	mat l e(V), format(%16.13g)
	global b1b2 = _b[d.tenure]
	mat omega = e(V)

gen fwage=lhwreal-_b[d.tenure]*tenure-_b[d.tenure2]*tenure2-_b[d.tenure3]*tenure3-_b[d.tenure4]*tenure4 /*
	*/ -_b[d.pexper2]*pexper2 -_b[d.pexper3]*pexper3-_b[d.pexper4]*pexper4-_b[d.logrankns]*logrankns-_b[d.lognsize]*lognsize /*
	*/ -_b[y3]*y3-_b[y4]*y4-_b[y5]*y5-_b[y6]*y6-_b[y7]*y7-_b[y8]*y8-_b[y9]*y9-_b[y10]*y10-_b[y11]*y11-_b[y12]*y12-_b[y13]*y13-_b[y14]*y14 /*
	*/ -_b[y15]*y15-_b[y15]*y15-_b[y16]*y16-_b[y17]*y17-_b[y18]*y18-_b[y19]*y19-_b[y20]*y20-_b[y21]*y21-_b[y22]*y22

* SECOND STEP ESTIMATION
reg fwage iniexp t0 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
mat l e(V), format(%16.13g)
	matrix omega1 = e(V)
	global b1 = _b[iniexp]
	global bt0 = _b[t0]
// Standard-error (Murphy & Topel, 1985)	
	quietly reg tenure iniexp t0 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
	mat l e(V), format(%16.13g)
		mat ZT = e(b)'
		mat l ZT, format(%16.13g)
	local vv = "tenure2 tenure3 tenure4 pexper2 pexper3 pexper4 logrankns lognsize y3 y4 y5 y6 y7 y8 y9 y10 y11 y12 y13 y14 y15 y16 y17 y18 y19 y20 y21 y22"
		foreach v of local vv {
		di _new "VAR:	`v'"_new
			quietly reg `v' iniexp t0 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
			mat l e(V), format(%16.13g)
				mat ZT = ZT,(e(b)')
				mat l ZT, format(%16.13g)
		}

mat help_matrix = ZT*omega*ZT'
mat l help_matrix, format(%16.13g)
	mat omega_total = omega1 + help_matrix
	mat l omega_total, format(%16.13g)
	matrix omega_total1 = omega_total[1,1]
	mat l omega_total1, format(%16.13g)
		di omega_total1[1,1]^.5
	mat topel_betasi = (($b1)\(omega_total1[1,1]^.5))
	mat l topel_betasi, format(%16.13g)
	matrix help_matrix1 = ZT*omega
	mat l help_matrix1, format(%16.13g)
di "Covariance Omega"				 

	matrix help_matrix2 = help_matrix1[1,1]
	matrix list help_matrix2, format(%16.13g)

	di _new "Return to tenure =    " $b1b2 - $b1 - $bt0 _new
	di _new "Standard-error =	" (omega_total[1,1] + omega[1,1] - 2 * help_matrix2[1,1])^.5

	mat topel_betas = (($b1b2 - $b1)\((omega_total[1,1] + omega[1,1] - 2 * help_matrix2[1,1])^.5))
	mat l topel_betas, format(%16.13g)
drop fwage

// IVs //

keep t0 dlhwreal dtenure2 dtenure3 dtenure4 dlogrankns dlognsize dpexper2 dpexper3 dpexper4 logrankns lognsize idtrab id idemp year lhwreal iniexp tenure tenure2 tenure3 tenure4 pexper pexper2 pexper3 pexper4 educ educ2 educ3 educ4 loc1 loc2 loc4 loc5 loc6 ind2-ind11 y1-y22

egen mtenure=mean(tenure),by(id)
	gen ivten=tenure-mtenure
egen mtenure2=mean(tenure2),by(id)
	gen ivten2=tenure2-mtenure2
egen mtenure3=mean(tenure3),by(id)
	gen ivten3=tenure3-mtenure3
egen mtenure4=mean(tenure4),by(id)
	gen ivten4=tenure4-mtenure4

egen mpexper=mean(pexper),by(idtrab)
	gen ivpexper=pexper-mpexper
egen mpexper2=mean(pexper2),by(idtrab)
	gen ivpexper2=pexper2-mpexper2
egen mpexper3=mean(pexper3),by(idtrab)
	gen ivpexper3=pexper3-mpexper3
egen mpexper4=mean(pexper4),by(idtrab)
	gen ivpexper4=pexper4-mpexper4

egen mlognsize=mean(lognsize),by(id)
	gen ivlognsize=lognsize-mlognsize

egen mlogrankns=mean(logrankns),by(id)
	gen ivlogrankns=logrankns-mlogrankns

	drop mtenure* mpexper* mlog*

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// # 5. //

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// # AS 1.
ivreg lhwreal (tenure tenure2 tenure3 tenure4 lognsize = ivten ivten2 ivten3 ivten4 ivlognsize) pexper pexper2 pexper3 pexper4 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12 y2-y22, cluster(idtrab)
xtreg lhwreal tenure tenure2 tenure3 tenure4 lognsize pexper pexper2 pexper3 pexper4 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12 y3-y22, fe cluster(idtrab)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// # 6. //

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// # AS 2.
ivreg lhwreal (tenure tenure2 tenure3 tenure4 logrankns lognsize = ivten ivten2 ivten3 ivten4 ivlogrankns ivlognsize) pexper pexper2 pexper3 pexper4 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12 y2-y22, cluster(idtrab)
xtreg lhwreal tenure tenure2 tenure3 tenure4 logrankns lognsize pexper pexper2 pexper3 pexper4 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12 y3-y22, fe cluster(idtrab)

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// # 7. //


drop ivten*


drop ivlog*
drop mtenure*
drop mlog*

egen mtenure2=mean(dtenure2),by(id)
	gen ivten2=dtenure2-mtenure2
egen mtenure3=mean(dtenure3),by(id)
	gen ivten3=dtenure3-mtenure3
egen mtenure4=mean(dtenure4),by(id)
	gen ivten4=dtenure4-mtenure4


egen  mlognsize=mean(dlognsize), by(id)
gen ivlognsize=dlognsize-mlognsize
egen mlogranks=mean(dlogranks), by(id)
gen ivlogranks=dlogranks-mlognrakns

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

di _new _new _new "TOPEL FULL; SPELL FE; LOGNSIZE" _new _new

* 1st stage with SPELL FE
	xtreg dlhwreal dtenure2 dtenure3 dtenure4 dlognsize dpexper2 dpexper3 dpexper4 y3-y22, fe cluster(idtrab)


ivreg dlhwreal (dtenure2 dtenure3 dtenure4 dlognsize = ivten2 ivten3 ivten4 ivlognsize) dpexper2 dpexper3 dpexper4 y3-Y22, cluster(idtrab)
	mat l e(V), format(%16.13g)
	global b1b2 = _b[_cons]
	mat omega = e(V)
	
gen fwage=lhwreal-_b[_cons]*tenure-_b[dtenure2]*tenure2-_b[dtenure3]*tenure3-_b[dtenure4]*tenure4 /*
	*/ -_b[dpexper2]*pexper2 -_b[dpexper3]*pexper3-_b[dpexper4]*pexper4-_b[dlognsize]*lognsize

foreach n of numlist 1983/2001 {
di _new _new "YEAR:	`n'"	_new
	local i = `n' - 1980
	di _new "i:	`i'" _new
	replace fwage = fwage -_b[y`i'] if year >= `n' & year <= 2001
}


* SECOND STEP ESTIMATION
reg fwage iniexp educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
mat l e(V), format(%16.13g)
	matrix omega1 = e(V)
	global b1 = _b[iniexp]

// Standard-error (Murphy & Topel, 1985)
	quietly reg tenure iniexp educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
	mat l e(V), format(%16.13g)
		mat ZT = e(b)'
		mat l ZT, format(%16.13g)	
	local vv = "tenure2 tenure3 tenure4 pexper2 pexper3 pexper4 lognsize y3 y4 y5 y6 y7 y8 y9 y10 y11 y12 y13 y14 y15 y16 y17 y18 y19 y20 y21 y22"
		foreach v of local vv {
		di _new "VAR:	`v'"_new
			quietly reg `v' iniexp educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
			mat l e(V), format(%16.13g)
				mat ZT = ZT,(e(b)')
				mat l ZT, format(%16.13g)
		}

mat help_matrix = ZT*omega*ZT'
mat l help_matrix, format(%16.13g)
	mat omega_total = omega1 + help_matrix
	mat l omega_total, format(%16.13g)
	matrix omega_total1 = omega_total[1,1]
	mat l omega_total1, format(%16.13g)
		di omega_total1[1,1]^.5
	mat topel_betasi = $b1\(omega_total1[1,1]^.5)
	mat l topel_betasi, format(%16.13g)
	matrix help_matrix1 = ZT*omega
	mat l help_matrix1, format(%16.13g)
di "Covariance Omega"

	matrix help_matrix2 = help_matrix1[1..2,1..2]
	matrix list help_matrix2, format(%16.13g)

	di _new "Return to tenure =    " $b1b2 - $b1 _new
	di _new "Standard-error =	" (omega_total[1,1] + omega_total[2,2] + omega[1,1] - 2 * help_matrix2[1,1] - 2 * help_matrix2[2,1] + 2*omega_total[2,1])^.5

drop fwage
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// # 8. //

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

di _new _new _new "TOPEL FULL; SPELL FE; USUAL PROCEDURE; WITH LOGRANKNS & LOGNSIZE" _new _new

* 1st stage with SPELL FE
	xtreg dlhwreal dtenure2 dtenure3 dtenure4 dlogrankns dlognsize dpexper2 dpexper3 dpexper4 y3-y22, fe cluster(idtrab)
	
ivreg dlhwreal (dtenure2 dtenure3 dtenure4 dlogrankns dlognsize = ivten2 ivten3 ivten4 ivlogrankns ivlognsize) dpexper2 dpexper3 dpexper4 y3-y22, cluster(idtrab)
	mat l e(V), format(%16.13g)
	global b1b2 = _b[_cons]
	mat omega = e(V)
gen fwage=lhwreal-_b[_cons]*tenure-_b[dtenure2]*tenure2-_b[dtenure3]*tenure3-_b[dtenure4]*tenure4 /*
	*/ -_b[dpexper2]*pexper2 -_b[dpexper3]*pexper3-_b[dpexper4]*pexper4-_b[dlogrankns]*logrankns-_b[dlognsize]*lognsize

foreach n of numlist 1983/2001 {
di _new _new "YEAR:	`n'"	_new
	local i = `n' - 1980
	di _new "i:	`i'" _new
	replace fwage = fwage -_b[y`i'] if year >= `n' & year <= 2001
}

* SECOND STEP ESTIMATION
reg fwage iniexp educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
mat l e(V), format(%16.13g)
	matrix omega1 = e(V)
	global b1 = _b[iniexp]

// Standard-error (Murphy & Topel, 1985)
	quietly reg tenure iniexp educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
	mat l e(V), format(%16.13g)
		mat ZT = e(b)'
		mat l ZT, format(%16.13g)
	local vv = "tenure2 tenure3 tenure4 pexper2 pexper3 pexper4 logrankns lognsize y3 y4 Y5 y6 y7 y8 y9 y10 y11 y12 y13 y14 Y15 y16 y17 y18 y19 y20 y21 y22"
		foreach v of local vv {
		di _new "VAR:	`v'"_new
			quietly reg `v' iniexp educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
			mat l e(V), format(%16.13g)
				mat ZT = ZT,(e(b)')
				mat l ZT, format(%16.13g)
		}

mat help_matrix = ZT*omega*ZT'
mat l help_matrix, format(%16.13g)
	mat omega_total = omega1 + help_matrix
	mat l omega_total, format(%16.13g)
	matrix omega_total1 = omega_total[1,1]
	mat l omega_total1, format(%16.13g)
		di omega_total1[1,1]^.5
	mat topel_betasi = $b1\(omega_total1[1,1]^.5)
	mat l topel_betasi, format(%16.13g)
	matrix help_matrix1 = ZT*omega
	mat l help_matrix1, format(%16.13g)
di "Covariance Omega"

	matrix help_matrix2 = help_matrix1[1..2,1..2]
	matrix list help_matrix2, format(%16.13g)

	di _new "Return to tenure =    " $b1b2 - $bt0 - $b1 _new
	di _new "Standard-error =	" (omega_total[1,1] + omega_total[2,2] + omega[1,1] - 2 * help_matrix2[1,1] - 2 * help_matrix2[2,1] + 2*omega_total[2,1])^.5

drop fwage

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

// TABLE VIII //

////////////////////////////////////////////////////

foreach n of numlist 1/10 {
di _new _new _new "STAGE:       `n'" _new _new
capture drop fwage

	bysort id (year): drop if _n==_N
 
// # 1. USUAL PROCEDURE
tsset id year
reg d.lhwreal d.tenure d.tenure2 d.tenure3 d.tenure4 d.pexper2 d.pexper3 d.pexper4 d.logrankns d.lognsize y3-y22, nocons
	mat l e(V), format(%16.13g)
	global b1b2 = _b[d.tenure]
	mat omega = e(V)

gen fwage=lhwreal-_b[d.tenure]*tenure-_b[d.tenure2]*tenure2-_b[d.tenure3]*tenure3-_b[d.tenure4]*tenure4 /*
	*/ -_b[d.pexper2]*pexper2 -_b[d.pexper3]*pexper3-_b[d.pexper4]*pexper4-_b[d.logrankns]*logrankns-_b[d.lognsize]*lognsize /*
        */ -_b[y3]*y3-_b[y4]*y4-_b[y5]*y5-_b[y6]*y6-_b[y7]*y7-_b[y8]*y8-_b[y9]*y9-_b[y10]*y10-_b[y11]*y11-_b[y12]*y12-_b[y13]*y13-_b[y14]*y14 /*
	*/ -_b[y15]*y15-_b[y15]*y15-_b[y16]*y16-_b[y17]*y17-_b[y18]*y18-_b[y19]*y19-_b[y20]*y20-_b[y21]*y21-_b[y22]*y22

* SECOND STEP ESTIMATION
reg fwage iniexp t0 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
mat l e(V), format(%16.13g)
	matrix omega1 = e(V)
	global b1 = _b[iniexp]
	global bt0 = _b[t0]
// Standard-error (Murphy & Topel, 1985)	
	quietly reg tenure iniexp t0 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
	mat l e(V), format(%16.13g)
		mat ZT = e(b)'
		mat l ZT, format(%16.13g)
	local vv = "tenure2 tenure3 tenure4 pexper2 pexper3 pexper4 logrankns lognsize y3 y4 y5 y6 y7 y8 y9 y10 y11 y12 y13 y14 y15 y16 y17 y18 y19 y20 y21 y22"
		foreach v of local vv {
		di _new "VAR:	`v'"_new
			quietly reg `v' iniexp t0 educ educ2 educ3 educ4 dumamt2-dumamt12 ind2-ind12, cluster(idtrab)
			mat l e(V), format(%16.13g)
				mat ZT = ZT,(e(b)')
				mat l ZT, format(%16.13g)
		}

mat help_matrix = ZT*omega*ZT'
mat l help_matrix, format(%16.13g)
	mat omega_total = omega1 + help_matrix
	mat l omega_total, format(%16.13g)
	matrix omega_total1 = omega_total[1,1]
	mat l omega_total1, format(%16.13g)
		di omega_total1[1,1]^.5
	mat topel_betasi = (($b1)\(omega_total1[1,1]^.5))
	mat l topel_betasi, format(%16.13g)
	matrix help_matrix1 = ZT*omega
	mat l help_matrix1, format(%16.13g)
di "Covariance Omega"				 

	matrix help_matrix2 = help_matrix1[1,1]
	matrix list help_matrix2, format(%16.13g)

	di _new "Return to tenure =    " $b1b2 - $b1 - $bt0 _new
	di _new "Standard-error =	" (omega_total[1,1] + omega[1,1] - 2 * help_matrix2[1,1])^.5

	mat topel_betas = (($b1b2 - $b1)\((omega_total[1,1] + omega[1,1] - 2 * help_matrix2[1,1])^.5))
	mat l topel_betas, format(%16.13g)
drop fwage

}

elapse `st'
log close
